/* ez-blob-blob.h: Prototype serialiser.

   Copyright (C) 2019 Michael Zucchi

   This program is free software: you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation, either version 3 of
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this program. If not, see
   <http://www.gnu.org/licenses/>.
*/

/*
 * The implemented serialisation mechanism only supports
 * forward-linked structures and will not perform well when
 * marshalling deeply nested structures but other serialisation
 * mechanisms are possible.
 *
 * It's historical and is to be removed.
 */

/**
 * Calculate the serialised blob size for a struct.
 *
 * @param d descriptor table matching struct.
 * @param p struct.
 */
size_t ez_blob_size(const ez_blob_desc *d, const void *p);

/**
 * Marshal a struct to a blob.
 * 
 * @param d descriptor table matching the struct.
 * @param p structure.
 * @param sizep return pointer for blob size, must not be null.
 * @param blob return, the data pointer must be freed with free() when complete.
 * @return non-zero on failure.
 */
int ez_blob_encode(const ez_blob_desc *d, const void *p, ez_blob *blob);

/**
 * Marshal a struct to a blob directly
 *
 * @param d descriptor table matching the struct.
 * @param p structure.
 * @param blob available target memory, size must be exactly ez_blob_size().
 * @return non-zero on failure.  On failure blob.data is invalid.
 */
int ez_blob_encode_raw(const ez_blob_desc *d, const void *p, ez_blob *blob);

/**
 * Unmarshall a blob into a struct.
 *
 * @param d descriptor table matching blob.
 * @param blob raw blob.
 * @param size blob size.
 * @return the decoded struct.  May be freed using ez_blob_free().
 */
void *ez_blob_decode(const ez_blob_desc *d, const ez_blob *blob);

/**
 * Decode to pre-allocated buffer.
 *
 * @param b blob data
 * @param size blob size
 * @return non-zero on failure.  Any allocated pointers are freed.
 */
int ez_blob_decode_raw(const ez_blob_desc *d, const ez_blob *blob, void *p);

